import 'package:dao_wrapped/dao_wrapped.dart';
import 'package:dao_wrapped/src/dao/dao_mixin.dart';
import 'package:dao_wrapped_annotation/dao_wrapped_annotation.dart';

/// uses the generated table from dao_wrapped_gen
mixin WithTable<T> on DatabaseGateway<T> {
  ModelTable get modelTable;

  String get table => modelTable.name;

  String get primaryId => modelTable.primaryKey;

  Future createTable() {
    return db.execute(createTableRequest());
  }

  String createTableRequest() {
    var _modelTable = modelTable;
    var createTableRequest = "";
    createTableRequest += "CREATE TABLE IF NOT EXISTS $table(";
    var index = -1;
    for (SqlField field in _modelTable.fields) {
      index++;
      createTableRequest += "${field.name} ${_describeEnum(field.type)} ";
      if (field.isPrimary) {
        createTableRequest += "PRIMARY KEY ";
      }
      if (field.autoIncrement) {
        createTableRequest += "AUTOINCREMENT ";
      }
      if (index < _modelTable.fields.length - 1) {
        createTableRequest += ",";
      }
    }
    createTableRequest += ")";
    return createTableRequest;
  }

  String _describeEnum(Object enumEntry) {
    final String description = enumEntry.toString();
    final int indexOfDot = description.indexOf('.');
    assert(indexOfDot != -1 && indexOfDot < description.length - 1);
    return description.substring(indexOfDot + 1);
  }
}
